@@ -21,7 +21,8 @@ static MISS: AtomicUsize = AtomicUsize::new(0);
 #[cfg(feature = "cache-buffer")]
 lazy_static! {
     static ref DEV_BUF_CACHE: Mutex<HashMap<usize, Vec<DeviceBuffer>>> = Mutex::new(HashMap::new());
-    static ref LOCKED_CACHE: Mutex<HashMap<usize, Vec<CudaLockedMemBuffer>>> = Mutex::new(HashMap::new());
+    static ref LOCKED_CACHE: Mutex<HashMap<usize, Vec<CudaLockedMemBuffer>>> =
+        Mutex::new(HashMap::new());
 }
 
 pub struct DeviceBuffer {
@@ -32,19 +33,37 @@ impl DeviceBuffer {
     pub fn new(len: usize) -> Self {
         #[cfg(feature = "cache-buffer")]
         {
-            let mut lock = DEV_BUF_CACHE.lock().unwrap();
+            let mut guard = DEV_BUF_CACHE.lock().unwrap();
 
-            let bufs = lock.entry(len).or_insert(Vec::new());
+            let bufs = guard.entry(len).or_insert(Vec::new());
             return match bufs.pop() {
                 Some(buf) => {
-                    let hit = HIT.fetch_add(1, Ordering::Relaxed) + 1;
-                    if hit % 1000 == 0 {
-                        let miss = MISS.load(Ordering::Relaxed);
-                        log::trace!("device buffer cache hit: {}, miss: {}", hit, miss);
+                    #[cfg(feature = "trace")]
+                    {
+                        let hit = HIT.fetch_add(1, Ordering::Relaxed) + 1;
+                        if hit % 1000 == 0 {
+                            let miss = MISS.load(Ordering::Relaxed);
+                            log::trace!("device buffer cache hit: {}, miss: {}", hit, miss);
+                        }
                     }
+
                     buf
                 }
-                None => Self::_new(len),
+                None => {
+                    #[cfg(feature = "trace")]
+                    {
+                        MISS.fetch_add(1, Ordering::Relaxed);
+                        log::trace!("miss device buffer for size: {}bytes", len);
+                        let stat = guard
+                            .iter()
+                            .map(|(k, v)| format!("{} * {}bytes", v.len(), k))
+                            .collect::<Vec<String>>()
+                            .join(",");
+                        log::trace!("current cached buffers: {}", stat);
+                    }
+
+                    Self::_new(len)
+                }
             };
         }
 
@@ -53,9 +72,6 @@ impl DeviceBuffer {
     }
 
     fn _new(len: usize) -> Self {
-        #[cfg(feature = "cache-buffer")]
-        MISS.fetch_add(1, Ordering::Relaxed);
-
         let inner = DevicePtr {
             ptr: unsafe { ffi::alloc_gpu_buffer(len) },
             len,
@@ -64,7 +80,7 @@ impl DeviceBuffer {
         Self { inner }
     }
 
-    fn _null() -> Self {
+    pub const fn null() -> Self {
         Self {
             inner: DevicePtr::null(),
         }
@@ -210,9 +226,10 @@ impl Drop for CudaLockedMemBuffer {
         #[cfg(feature = "cache-buffer")]
         {
             let mut lock = LOCKED_CACHE.lock().unwrap();
-            lock.entry(self.len())
-                .or_insert(Vec::new())
-                .push(Self { inner: self.inner, size: self.size });
+            lock.entry(self.len()).or_insert(Vec::new()).push(Self {
+                inner: self.inner,
+                size: self.size,
+            });
         }
 
         #[cfg(not(feature = "cache-buffer"))]
@@ -239,9 +256,7 @@ impl<T> From<&[T]> for CudaLockedMemBuffer {
         let len = std::mem::size_of::<T>() * v.len();
         let mut buf = Self::new(len);
 
-        let src = unsafe {
-            std::slice::from_raw_parts(v.as_ptr() as *const u8, len)
-        };
+        let src = unsafe { std::slice::from_raw_parts(v.as_ptr() as *const u8, len) };
 
         buf.as_mut().copy_from_slice(src);
 
@@ -249,15 +264,12 @@ impl<T> From<&[T]> for CudaLockedMemBuffer {
     }
 }
 
-
 impl<T> From<Vec<T>> for CudaLockedMemBuffer {
     fn from(v: Vec<T>) -> Self {
         let len = std::mem::size_of::<T>() * v.len();
         let mut buf = Self::new(len);
 
-        let src = unsafe {
-            std::slice::from_raw_parts(v.as_ptr() as *const u8, len)
-        };
+        let src = unsafe { std::slice::from_raw_parts(v.as_ptr() as *const u8, len) };
 
         buf.as_mut().copy_from_slice(src);
 
@@ -293,7 +305,7 @@ impl DevicePtr {
         }
     }
 
-    pub fn null() -> Self {
+    pub const fn null() -> Self {
         Self {
             ptr: std::ptr::null(),
             len: 0,
